﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Resources;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;
using System.Windows.Media;

namespace HelloWorld
{
    public partial class Page : UserControl
    {
        public Page()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Page_Loaded);
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapTools.MouseCoordinate.MouseCoordinateType = MouseCoordinateType.LatitudeLongitude;
            Map1.MapTools.PanZoomBar.IsEnabled = true;
            Map1.MapUnit = GeographyUnit.DecimalDegree;
            Map1.MapTools.Logo.IsEnabled = false;
            Map1.MapTools.MouseCoordinate.IsEnabled = false;
            Map1.Background = new SolidColorBrush(Color.FromArgb(255, 47, 45, 45));
            CreateZoomLevels(14);
            var extent = new RectangleShape(new PointShape(77.68166,12.9337), new PointShape(77.69404,12.91624));
           
         
            ProjNetProjection projNetProjection = new ProjNetProjection(4326, 4326);
            projNetProjection.Open();

            Map1.CurrentExtent = projNetProjection.ConvertToExternalProjection(extent);
            

            Map1.CurrentExtent = projNetProjection.ConvertToExternalProjection(extent);
            var overlay = new ServerLayerOverlay("HTSL_1.jp2", "SilverlightMapConnector1");
           
            Map1.Overlays.Add(overlay);         
        }

        private void CreateZoomLevels(int configuredZoomLevelID)
        {
            const int totalNumberOfZoomLevelsTobeConfigured = 19;//Configured value starts from 0 till 19.Hence the constant is 19
            const string functionName = "CreateZoomLevels";
            try
            {
                Map1.ZoomLevelSet.CustomZoomLevels.Clear();
                var upperZoomScale = Map1.ZoomLevelSet.ZoomLevel20.Scale;
                var lowerZoomScale = Map1.ZoomLevelSet.ZoomLevel01.Scale;
                var configuredZoomScale = GetConfiguredZoomScale(lowerZoomScale, configuredZoomLevelID);
                var zoomLevelSet = new ZoomLevelSet();
                var intialZoomScale = configuredZoomScale;
                int stepsNeededinBetweenScales = 0;
                double stepScale = 0;
                var availableZoomLevels = totalNumberOfZoomLevelsTobeConfigured - configuredZoomLevelID;
                //This many number of zoom levels are available for us from configuration
                if (configuredZoomLevelID != 0) //Recreate zoom levels 
                {
                    if (availableZoomLevels != 0)
                    {
                        if (availableZoomLevels != 1)
                            stepsNeededinBetweenScales = (totalNumberOfZoomLevelsTobeConfigured - availableZoomLevels) /
                                                         (availableZoomLevels - 1);
                        //Only available availableZoomLevels - 1 levels 
                        //are there for inserting step zoom level
                        else
                            stepsNeededinBetweenScales = totalNumberOfZoomLevelsTobeConfigured - 1;
                        // In this case we have only one step to insert all zoom levels.
                    }
                    zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(configuredZoomScale));
                    for (int majorZoomLevel = 1; majorZoomLevel < availableZoomLevels; majorZoomLevel++)
                    {
                        for (int step = 1; step <= stepsNeededinBetweenScales; step++)
                        {
                            var lowerStepScale = intialZoomScale;
                            var upperStepScale = intialZoomScale / 2;
                            stepScale = (lowerStepScale - upperStepScale) / (stepsNeededinBetweenScales + 1);
                            // Since we are adding major zoom levels in upper loop
                            // adding 1 to stepsNeededinBetweenScales for calculation
                            if (zoomLevelSet.CustomZoomLevels.Count >= totalNumberOfZoomLevelsTobeConfigured - 1)
                                break; // If at all the count  goes more than 18 due to round off break the loop
                            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(lowerStepScale - stepScale * step));
                        }
                        intialZoomScale = intialZoomScale / 2;
                        zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(intialZoomScale));
                    }
                    //Due to round off while calculting the number of stepsneeded inbetween, we will loose some zoom levels or get some zoom levels
                    if (zoomLevelSet.CustomZoomLevels.Count < totalNumberOfZoomLevelsTobeConfigured)
                    {
                        var deficitLevels = totalNumberOfZoomLevelsTobeConfigured - zoomLevelSet.CustomZoomLevels.Count;
                        var scaleToBeadded = (intialZoomScale - upperZoomScale) / (deficitLevels + 1);
                        // adding one to ensure that we can yet have one more last zoom level to be added
                        //We need to increase some steps because of round off while calculating number of steps needed in between
                        for (int step = 1; step <= deficitLevels; step++)
                            zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(intialZoomScale - step * scaleToBeadded));
                    }
                    zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(upperZoomScale));// This is the last scale for the map

                }
                else //If map is configured with 0 zoom level,add only default zoom levels
                {
                    for (int majorZoomLevel = configuredZoomLevelID;
                         majorZoomLevel <= totalNumberOfZoomLevelsTobeConfigured;
                         majorZoomLevel++)
                    {
                        intialZoomScale = intialZoomScale / 2;
                        zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(intialZoomScale));
                    }
                }
                Map1.ZoomLevelSet = zoomLevelSet;
            }
            catch (Exception ex)
            {

                throw;
            }
        }
        /// <summary>
        /// This function gets configured zoom scale depending on the zoom level ID saved in DB
        /// </summary>
        /// <param name="maxZoomScale"></param>
        /// <param name="configuredZoomLevelID"></param>
        /// <returns></returns>
        private double GetConfiguredZoomScale(double maxZoomScale, int configuredZoomLevelID)
        {
            double factorToDevide = Math.Pow(2, (configuredZoomLevelID));
            double configuredZoomScale = maxZoomScale / factorToDevide;
            return configuredZoomScale;
        }
        // Load Shape files from resource.
     
    }
}